
(* SPAN - A Security Protocol Animator
   Copyright (C) 2005-2006 Yann Glouche and Thomas Genet

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.0. Last modification 30/11/06
*)
open Tk;;
(**
utils_simulator.ml
@author Yann GLOUCHE
@version 1.0
@since 12/12/2005
*)

(** graphicals objects *)

(** Initialize Tk *)
let root = (openTk () : Widget.toplevel Widget.widget) ;;

(** Colors *)

let defcol = (`Color "#dfdfdf" : [> `Color of string]);;
let selcol = (`Color "#ffffff" : [> `Color of string]);;
let blue = (`Color "#0000ff" : [> `Color of string]);;
let black = (`Color "#000000" : [> `Color of string]);;
let red = (`Color "#ff0000" : [> `Color of string]);;
let green = (`Color "#006600" : [> `Color of string]);;
let blue_light = (`Color "#9999ff" : [> `Color of string]);;
let clicol = (`Color "#ffafaf" : [> `Color of string]);;
let violet = (`Color "#770077" : [> `Color of string]);;
let orange = (`Color "#FFAA00" : [> `Color of string]);;
let violet_light = (`Color "#FF00FF" : [> `Color of string]);;

(** Text font *)

let get_font (tag : string) (choices : string list) : Tk.font =
  let all = Font.families () in
  let rec find ch =
    match ch with
    | [] -> ""
    | f::tail ->
      ( try (List.find (( = ) f) all)
        with Not_found -> find tail ) in
  match find choices with
  | "" -> Font.create ~name:tag ~size:10 ()
  | fam -> Font.create ~name:tag ~family:fam ~size:10 ();;

let configure () : unit =
  ignore(get_font "guifont" ["verdana"; "lucida"; "bitstream vera sans"]);
  ignore(get_font "textfont"     ["lettergothic"; "lucida typewriter"; "bitstream vera sans mono"; "courier"]);
  Option.add ~path:"*Menu.font" "guifont";
  Option.add ~path:"*Label.font" "guifont";
  Option.add ~path:"*Message.font" "guifont";
  Option.add ~path:"*Button.font" "guifont";
  Option.add ~path:"*Radiobutton.font" "guifont";
  Option.add ~path:"*Checkbutton.font" "guifont";
  Option.add ~path:"*Text.font" "textfont";
  Option.add ~path:"*Canvas.font" "textfont";
  Option.add ~path:"*Listbox.font" "textfont";
  Option.add ~path:"*Entry.font" "textfont";
  Option.add ~path:"*Text.borderWidth" "1";
  Option.add ~path:"*Canvas.borderWidth" "1";
  Option.add ~path:"*Button.borderWidth" "1";
  Option.add ~path:"*Scrollbar.borderWidth" "1";
  Option.add ~path:"*Entry.borderWidth" "1";
  Option.add ~path:"*Listbox.borderWidth" "1";
  Option.add ~path:"*Menu.borderWidth" "1";
  Option.add ~path:"*Menu.activeBorderWidth" "1";
  Option.add ~path:"*Menu.tearOff" "false";
  Option.add ~path:"*Listbox.selectBorderWidth" "0";
  Option.add ~path:"*Listbox.selectBackground" "#0000a0";
  Option.add ~path:"*Listbox.selectForeground" "#ffffff";
  Option.add ~path:"*Listbox.background" "#ffffff";;

let words = ([
  "anaerobic"; "beneficent"; "corundum"; "diesel";
  "erotic"; "fragment"; "garrulous"; "hexagon";
  "irate"; "jinx"; "kerosene"; "lethargic";
  "malamute"; "normality"; "oblate"; "parsimony";
  "quail"; "rigorous"; "scimitar"; "tuxedo";
  "ululation"; "verify"; "warbler"; "xenon";
  "yawl"; "zither"
] : string list);;

let nop (() : unit) : unit = ();;

configure () ;;

(** 
scroll_link_canvas link scroll event of the canvas to the scroll bar
@param sb scroll bar
@param lst canvas
@param orientation the value "horizontal" is for the horizontal scroll bar, the value "vertical" is for the vertical scroll bar
@return unit
*)
let scroll_link_canvas (sb : Widget.scrollbar Widget.widget)  (lst : Widget.canvas Widget.widget) (orientation : string) : unit =
    if(orientation="horizontal") then(
    	Canvas.configure ~xscrollcommand:(Scrollbar.set sb;) lst; 
	Scrollbar.configure ~command:(Canvas.xview lst) sb )
    else (Canvas.configure ~yscrollcommand:(Scrollbar.set sb) lst;
    	  Scrollbar.configure ~command:(Canvas.yview lst) sb);;
	  
(** 
scroll_link link scroll event of the canvas to the scroll bar
@param sb scroll bar
@param lst list box
@param orientation the value "horizontal" is for the horizontal scroll bar, the value "vertical" is for the vertical scroll bar
@return unit
*)
let scroll_link (sb : Widget.scrollbar Widget.widget)  (lst : Widget.listbox Widget.widget) (orientation : string) : unit =
    if(orientation="horizontal") then(
    	Listbox.configure ~xscrollcommand:(Scrollbar.set sb) lst; 
	Scrollbar.configure ~command:(Listbox.xview lst) sb )
    else (Listbox.configure ~yscrollcommand:(Scrollbar.set sb) lst;
    	  Scrollbar.configure ~command:(Listbox.yview lst) sb);;
	  
(** list treatments *)
	   
(**
remove an element "elt" from a list "l"
@param elt an element
@param l a list
@return 'a list without the element elt
*)
let remove (elt : 'a) (l : 'a list) : 'a list =
	let rec aux l res =
		match l with [] -> List.rev res
			|(x::ls) -> 	if (elt=x)
					then aux ls res
					else aux ls (x::res);
	in
	aux l [];;
	
(**
remove just one occurence of an element "elt" from a list "l"
@param elt an element
@param l a list
@return 'a list without the element elt
*)
let remove_one (elt : 'a) (l : 'a list) : 'a list =
	let rec aux l res =
		match l with [] -> List.rev res
			|(x::ls) -> 	if (elt=x)
					then (List.rev res)@ls
					else aux ls (x::res);
	in
	aux l []
;;
(**
remove just the element to the index "index" from a list "l"
@param elt an element
@param l a list
@return 'a list without the element elt
*)
let remove_index (index : int) (l : 'a list) : 'a list =
	let rec aux index l res =
		match l with [] -> List.rev res
			|(x::ls) -> 	if (index=0)
					then (List.rev res)@ls
					else aux (index-1) ls (x::res);
	in
	aux index l []
;;
(**
remove the multiple occurences of the element in a list "l"
@param elt an element
@return 'a list without the multiple occurences of the elements
*)	
let remove_double (l : 'a list) : 'a list =
	let rec aux l res =
		match l with [] -> List.rev res
			|(x::ls) -> 	if (List.mem x res)
					then aux ls res
					else aux ls (x::res);
	in
	aux l[];;

(** test if the string "s1" contains the string "s2"
@param s2 string
@param s1 string
@return bool
*)
let containStr (s2 : string) (s1 : string) : bool =
 let rec inString s1 s2 l1 l2 =(
 if (l1 < l2) then false
 else(if(l1 = l2) then (s1=s2)
      else(if((String.sub s1 0 l2)=s2)
           then true
           else inString (String.sub s1 1 (l1-1)) s2 (l1-1) l2))) in
 let l1 = String.length s1 in
 let l2 =  String.length s2 in
 inString s1 s2 l1 l2;;

(** return the position of the string "s2" in the string "s1". return -1, if the string "s1" doesn't contain the string "s2"
@param s2 string
@param s1 string
@return int
*)
let indexStr (s2 : string) (s1 : string) : int =
 let rec inString s1 s2 l1 l2 =(
 if (l1 < l2) then -1
 else(if(l1 = l2) then (if (s1=s2) 
			then l1 
			else -1)
      else(if((String.sub s1 0 l2)=s2)
           then l1
           else inString (String.sub s1 1 (l1-1)) s2 (l1-1) l2))) in
 let l1 = String.length s1 in
 let in_string = (inString s1 s2 l1 (String.length s2) ) in
 if (in_string=(-1))
 then (-1)
 else (l1-in_string);;
 (**
 find a separator "k" in a string "message" "(ak...kb)". It is used for find the arguments in a string of the form "a(b,..,c)"  whitk k=","
 @param sep is a separator. for example : ","
 @param k int
 @param message string
 @return int, a position
 *)
 let find_separator (sep : string) (k : int) (message : string) : int = 
	let l = String.length message in
	let rec find_separator_aux i res =
		if(res<l)
		then(
			let tmp = (String.get message res) in
			if(i=k && tmp=sep.[0]) then (res)
			else if (tmp='(') then (find_separator_aux (i+1) (res+1))
			else if(tmp=')') then (find_separator_aux (i-1) (res+1))
			else(find_separator_aux i (res+1)); 
		)
		else (-1);
		in
 	(find_separator_aux 0 0) ;;
(**
replace a string "s1" by the string "s2" in a string "s"
@param s1 is a string
@param s2 is a string
@param s is a string
@return a new string
*)
let rec replace (s1 : string) (s2 : string) (s : string) =
	let l_s1 = String.length s1 in
	let l_s2 = String.length s2 in
	let rec replace_aux s pos =
		let l_s = String.length s in
		if(l_s1>l_s) then s
		else if(l_s1=l_s && s1=s) then s2
		else(
			let l_s = String.length s in
			let tmp_pos = indexStr  s1 (String.sub s pos (l_s-(pos+0)) ) in 
			let new_pos = pos+tmp_pos in
			if(tmp_pos<>(-1))
			then(
				if(new_pos > pos)
				then replace_aux ((String.sub s 0 new_pos)^s2^(String.sub s (new_pos+l_s1) (l_s-new_pos-l_s1) )) (new_pos+l_s2)
				else ((String.sub s 0 new_pos)^s2^(String.sub s (new_pos+l_s1) (l_s-new_pos-l_s1) ))
			)
			else s
		)
	in 
	replace_aux s 0
;;	
(** decompose a string of the form "a(b,...,c)" ina string list of the form [a ;b ... ; c]
@param m1 is a string of the form "a(b,...,c)"
@return a string list of the form [a ;b ... ; c]
*)
 let decompose (m1 : string) : string list =
	let l = String.length m1 in
	let index_brack = indexStr "(" m1 in
	if(index_brack<>(-1) && (String.sub m1 (l-1) 1)=")")
	then(
	     let tmp = String.sub m1 ((index_brack)+1) (l-index_brack-2) in
	     let l2 = l-index_brack-2 in
	     let rec aux l_tmp tmp res = 
	     	let separator = find_separator "," 0 tmp in
		if(separator<>(-1))
		then (aux  (l_tmp-separator-1) (String.sub tmp (separator+1) (l_tmp-separator-1)) ((String.sub tmp 0 separator)::res))
		else ((String.sub m1 0 index_brack)::((List.rev res)@[tmp]))
	     in
	     if(tmp<>"")
	     then aux l2 tmp []
	     else [(String.sub m1 0 index_brack)];
	     )
	 else [];;
	 
(** 
@param l a 'a list
@return the head of the list "l"
*)
 let car (l : 'a list) : 'a = match l with (x::_) -> x
 		   			|_ -> failwith "empty list";;
 (** 
@param l a 'a list
@return the tail of the list "l"
*)
 let cdr (l : 'a list) : 'a list = match l with (_::ls) -> ls
 		  		 	|_ -> failwith "no more element";;

(** 
@param l a 'a list
@return the second element of the list "l"
*)			
 let cadr (l : 'a list) : 'a  = match l with (_::x::_) -> x
 		   			|_ -> failwith "empty list";;
 (** remove two elements at the begining to the list
@param l a 'a list
@return the list "l" without two elements at the begining to the list
*)
 let cddr (l : 'a list) : 'a list = match l with (_::_::ls) -> ls
 				   	|_ -> failwith "no more element";;

(** 
@param l a 'a list
@return the third of the list "l"
*)		
 let caddr (l : 'a list) : 'a  = match l with (_::_::x::_) -> x
 		   			|_ -> failwith "empty list";;

(**
remove the blank after the position "i" until another character in a string "s" with a length "l"
@param s string
@param l in, is a length
@param i int, is position in the string "s"after the position "i" until another character in a string "s" with a length "l"
@return a string
*)
 let rec remove_left_space (s : string) (l : int) (i : int) : string = (
   if (i<l) 
   then (if ((String.get s i)=' ') 
   	then (remove_left_space s l (i+1)) 
	else (String.sub s i (l-i));) 
   else "";
  ) ;;
  (**
remove the blank before the position "i" until another character in a string "s" with a length "l"
@param s string
@param l in, is a length
@param i int, is position in the string "s"
@return a string
*)
 let rec remove_right_space (s : string) (l : int) (i : int) : string = (
   if (i<l) 
   then (if ((String.get s (l-i-1))=' ') 
   	then (remove_right_space s l (i+1)) 
	else (String.sub s 0 (l-i));) 
   else "";
  ) ;;
 (**
remove the blank at the beginning of a string "s" and at the end of this string "s"
@param s string
@return a string
*)
let remove_space (s : string) : string = (
  let l = String.length s in
  let tmp = remove_right_space s l 0 in
  let l2 = String.length tmp in
  remove_left_space tmp l2 0;
 );;
 
(**
decompose a string "mess" of the form "a -> b | c" in a 3-uplet (a,b,c)
@param mess is a string of the form "a -> b | c"
@return a 3-uplet (a,b,c) where the type of a, b, and c is string
*)
let actionTreatment (mess : string) : string * string * string =
 let sender = remove_space (String.sub mess 0 (String.index mess '-')) in
 let mark1 = ((String.index mess '>')+1) in
 let reciever = remove_space (String.sub mess mark1 ((String.index mess ':')-mark1)) in
 let mark2 =  ((String.index mess ':')+1) in
 let message = remove_space(String.sub mess mark2 ((String.length mess)-mark2)) in
 (sender, reciever, message);;
 
 (**
 round a float to the near integer value
 @param y float
 @return a new float
 *)
 let  round (y : float) : float = 
	let floor_y = floor y in
	if((abs_float(y -. floor_y)) > 0.5) then (floor_y +. 1.) else floor_y
;;

(** compute the min in a list of int, float, double 
@param l is a list of numbers
@return a number
*)
 let min_l (l : 'a list) : 'a = List.fold_left min (car l) (cdr l);;
 (** compute the max in a list of int, float, double 
@param l is a list of numbers
@return a number
*)
 let max_l (l : 'a list) : 'a = List.fold_left max (car l) (cdr l);;
 let list2string (sep : string) (l : string list) : string = 
 	if(l=[]) then""
	else(
 		let res = List.fold_left (fun a b -> a^sep^b) "" l in
		let length = String.length sep in
		if (length>0)
		then String.sub res length ((String.length res)-length)
		else res
	)	
 ;;

 (** return the name of a role composed a number of instanciation
 @param name is the name of a role
 @param id is the number of instantiation of the role "name"
 @return string, the name of a role composed a number of instanciation
 *)
 let get_name_in_session (name : string) (id : int) : string = "("^name^", "^(string_of_int id)^")";;

 (** return the maximal length of a string in a list of strings 
 @param l is a list of strings
 @return the maximal length
 *)
 let max_length_string (l : string list) : int = 
 	match l with [] -> 0
		|[x] -> String.length x
		|_ -> max_l (List.map String.length l);;
 
